/*
Module : SelectCertificateDlg.cpp
Purpose: Implementation for a MFC dialog class to allow selection of cryptography certificates
Created: PJN / 27-08-2006
History: PJN / 05-07-2007 1. Updated copyright messages.
                          2. Updated the sample app to clean compile on VC 2005
                          3. Addition of a CSELECTCERTIFICATEDLG_EXT_CLASS preprocessor macro which makes 
                          the class easier to use in an extension dll. 
         PJN / 28-12-2008 1. Updated copyright details
                          2. Removed VC 6 style AppWizard comments from the code
                          3. The code has now been updated to support VC 2005 or later only
                          4. Code now compiles cleanly using Code Analysis (/analyze)
                          5. Updated code to compile correctly using _ATL_CSTRING_EXPLICIT_CONSTRUCTORS define
                          6. Added explicit support for the following OID's:                          
                          szOID_PKIX_KP_OCSP_SIGNING, szOID_PKIX_OCSP_NOCHECK, szOID_PKIX_OCSP_NONCE,
                          szOID_KP_CTL_USAGE_SIGNING, szOID_SERVER_GATED_CRYPTO, szOID_SGC_NETSCAPE,
                          szOID_KP_MOBILE_DEVICE_SOFTWARE, szOID_KP_SMART_DISPLAY, szOID_KP_CSP_SIGNATURE,
                          szOID_DRM_INDIVIDUALIZATION & szOID_KP_KERNEL_MODE_CODE_SIGNING
         PJN / 03-11-2012 1. Updated copyright details.
                          2. Updated the sample app to open the system certificate store in read only mode. 
                          This avoids access denied errors on Windows Vista and later.
                          3. Code no longer uses LoadLibrary without an absolute path when loading cryptui DLL. 
                          This avoids DLL planting security issues.
         PJN / 18-01-2015 1. Updated copyright details. 
                          2. Updated the code and sample app to clean compile on VC 2010 - VC 2013
         PJN / 19-11-2015 1. Verified the code compiles cleanly on VC 2015.
                          2. Updated the sample apps main icon.
                          3. Updated copyright details.
                          4. CryptUIDlgViewCertificate is now referenced via import library rather than 
                          by using GetProcAddress.
                          5. Separator used for "Intended Use" column is now setup by calling GetLocaleInfo.
         PJN / 01-05-2017 1. Updated copyright details.
                          2. Updated the code to compile cleanly using /permissive-.
         PJN / 23-09-2018 1. Updated copyright details.
                          2. Fixed a number of C++ core guidelines compiler warnings. These changes mean that
                          the code will now only compile on VC 2017 or later.
         PJN / 21-05-2019 1. Updated copyright details.
                          2. Updated the code to clean compile on VC 2019
         PJN / 12-11-2019 1. Updated initialization of various structs to use C++ 11 list initialization
                          2. Replaced BOOL with bool in a number of places throughout the codebase
         PJN / 29-12-2019 1. Fixed various Clang-Tidy static code analysis warnings in the code.
         PJN / 27-03-2020 1. Updated copyright details.
                          2. Fixed more Clang-Tidy static code analysis warnings in the code.
                          3. Reimplemented m_Certificates & m_CertificateSelectionState arrays
                          using std::vector.
         PJN / 10-05-2020 1. Fixed more Clang-Tidy static code analysis warnings in the code.
         PJN / 16-04-2022 1. Updated copyright details.
                          2. Updated the code to use C++ uniform initialization for all variable declarations

Copyright (c) 2006 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"
#include "SelectCertificateDlg.h"
#ifndef __AFXPRIV_H__
#pragma message("To avoid this message, please put afxpriv.h in your pre compiled header (normally stdafx.h)")
#include <afxpriv.h> //WM_KICKIDLE support
#endif //#ifndef __AFXPRIV_H__
#ifndef __CRYPTUIAPI_H__
#pragma message("To avoid this message, please put cryptuiapi.h in your pre compiled header (normally stdafx.h)")
#include <cryptuiapi.h>
#endif //#ifndef __CRYPTUIAPI_H__


//////////////////// Macros / Defines /////////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG

#pragma comment(lib, "Crypt32.lib")
#pragma comment(lib, "Cryptui.lib")

#ifndef szOID_PKIX_KP_OCSP_SIGNING
#define szOID_PKIX_KP_OCSP_SIGNING "1.3.6.1.5.5.7.3.9"
#endif //#ifndef szOID_PKIX_KP_OCSP_SIGNING

#ifndef szOID_PKIX_OCSP_NOCHECK
#define szOID_PKIX_OCSP_NOCHECK "1.3.6.1.5.5.7.48.1.5"
#endif //#ifndef szOID_PKIX_OCSP_NOCHECK

#ifndef szOID_PKIX_OCSP_NONCE
#define szOID_PKIX_OCSP_NONCE		"1.3.6.1.5.5.7.48.1.2"
#endif //#ifndef szOID_PKIX_OCSP_NONCE

#ifndef szOID_KP_SMART_DISPLAY
#define szOID_KP_SMART_DISPLAY "1.3.6.1.4.1.311.10.3.15"
#endif //#ifndef szOID_KP_SMART_DISPLAY

#ifndef szOID_KP_CSP_SIGNATURE
#define szOID_KP_CSP_SIGNATURE "1.3.6.1.4.1.311.10.3.16"
#endif //#ifndef szOID_KP_CSP_SIGNATURE

#ifndef szOID_KP_KERNEL_MODE_CODE_SIGNING
#define szOID_KP_KERNEL_MODE_CODE_SIGNING "1.3.6.1.4.1.311.61.1.1"
#endif //#ifndef szOID_KP_KERNEL_MODE_CODE_SIGNING


//////////////////// Implementation ///////////////////////////////////////////

#pragma warning(suppress: 26455)
CSelectCertificateDlg::CSelectCertificateDlg(_In_opt_ CWnd* pParent) : CDialog{CSelectCertificateDlg::IDD, pParent},
                                                                       m_bExcludeIssuedToColumn{false},
                                                                       m_bExcludeIssuedByColumn{false},
                                                                       m_bExcludeIntendedPurposesColumn{false},
                                                                       m_bExcludeFriendlyNameColumn{false},
                                                                       m_bExcludeExpirationColumn{false},
                                                                       m_hCertStore{nullptr},
                                                                       m_dwViewFlags{CRYPTUI_DISABLE_ADDTOSTORE | CRYPTUI_DISABLE_EDITPROPERTIES},
                                                                       m_bAllowMultipleSelection{false},
                                                                       m_bHideViewCertificateButton{false},
                                                                       m_pInitialSelectedCertificate{nullptr}
{
}

CSelectCertificateDlg::~CSelectCertificateDlg()
{
  //Free up the cert contexts
  for (auto& certificate : m_Certificates)
    CertFreeCertificateContext(certificate);
}

INT_PTR CSelectCertificateDlg::GetCertificatesShownCount() const noexcept
{
  return m_Certificates.size();
}

#pragma warning(suppress: 26440)
PCCERT_CONTEXT CSelectCertificateDlg::GetCertificateShown(_In_ INT_PTR nIndex, _Out_ bool& bWasSelected)
{
  //Get the selection state
#pragma warning(suppress: 26446)
  bWasSelected = m_CertificateSelectionState[nIndex];

  //Get the actual certificate, note we duplicate the cert context so client code
  //is responsible for freeing it using "CertFreeCertificateContext"
#pragma warning(suppress: 26446)
  return CertDuplicateCertificateContext(m_Certificates[nIndex]);
}

#pragma warning(suppress: 26440)
PCCERT_CONTEXT CSelectCertificateDlg::GetNextSelectedCertificateShown(_Inout_ int& nIndex)
{
#pragma warning(suppress: 26472)
  const int nCertificates{static_cast<int>(m_CertificateSelectionState.size())};
  for (int i{nIndex}; i<nCertificates; i++)
  {
    //Have we found the next selected cert?
#pragma warning(suppress: 26446)
    if (m_CertificateSelectionState[i])
    {
      nIndex = i;
#pragma warning(suppress: 26446)
      return CertDuplicateCertificateContext(m_Certificates[i]);
    }
  }

  //Fail the search
  nIndex = -1;
  return nullptr;
}

bool CSelectCertificateDlg::SetInitialSelectedCert(_In_ PCCERT_CONTEXT pCertContext) noexcept
{
  //Free up any existing cert
  if (m_pInitialSelectedCertificate != nullptr)
  {
    CertFreeCertificateContext(m_pInitialSelectedCertificate);
    m_pInitialSelectedCertificate = nullptr;
  }

  //Save our copy of the cert context
  m_pInitialSelectedCertificate = CertDuplicateCertificateContext(pCertContext);
  return (m_pInitialSelectedCertificate != nullptr);
}

void CSelectCertificateDlg::DoDataExchange(CDataExchange* pDX)
{
  __super::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_SELECT_CERT_PROMPT, m_ctrlPrompt);
  DDX_Control(pDX, IDC_SELECT_CERT_LIST, m_ctrlList);
}

#pragma warning(suppress: 26440 26433)
BEGIN_MESSAGE_MAP(CSelectCertificateDlg, CDialog) //NOLINT(modernize-avoid-c-arrays)
  ON_WM_DESTROY()
  ON_MESSAGE(WM_KICKIDLE, &CSelectCertificateDlg::OnKickIdle)
  ON_BN_CLICKED(IDC_VIEW_CERTIFICATE, &CSelectCertificateDlg::OnViewCertificate)
  ON_UPDATE_COMMAND_UI(IDC_VIEW_CERTIFICATE, &CSelectCertificateDlg::OnUpdateViewCertificate)
END_MESSAGE_MAP()

void CSelectCertificateDlg::UpdateCaptionAndPromptStrings()
{
  //Update the caption and prompt strings
  if (m_sTitle.GetLength())
    SetWindowText(m_sTitle);
  if (m_sDisplayString.GetLength())
    m_ctrlPrompt.SetWindowText(m_sDisplayString);
}

void CSelectCertificateDlg::SetupListControl()
{
  //Hook up the image list to the list control
  m_ilCertificates.Create(IDB_SELECT_CERTIFICATE_IMAGELIST, 16, 1, RGB(255, 0, 255));
  m_ctrlList.SetImageList(&m_ilCertificates, LVSIL_SMALL);

  //Turn on full row select and label tips
  m_ctrlList.SetExtendedStyle(LVS_EX_FULLROWSELECT | LVS_EX_LABELTIP);

  //turn on single or multi select
  if (m_bAllowMultipleSelection)
    m_ctrlList.ModifyStyle(LVS_SINGLESEL, 0);
  else
    m_ctrlList.ModifyStyle(0, LVS_SINGLESEL);
}

void CSelectCertificateDlg::AddCertificateColumns()
{
  //First need to work out the column width for the list control
  CRect rList;
  m_ctrlList.GetClientRect(&rList);
  int nColumnsToShow{5};
  if (m_bExcludeIssuedToColumn)
    --nColumnsToShow;
  if (m_bExcludeIssuedByColumn)
    --nColumnsToShow;
  if (m_bExcludeIntendedPurposesColumn)
    --nColumnsToShow;
  if (m_bExcludeFriendlyNameColumn)
    --nColumnsToShow;
  if (m_bExcludeExpirationColumn)
    --nColumnsToShow;
  ASSERT(nColumnsToShow != 0); //It doesn't make sense to exclude all the columns!!!
  if (nColumnsToShow == 0)
    return;
  const int nColumnWidth{(rList.Width() - 16) / nColumnsToShow}; //Allow 16 extra pixels for the first column

  //Now add each of the columns
  int nColumn{0};
  CString sColumnHeading;
  if (!m_bExcludeIssuedToColumn)
  {
    if (sColumnHeading.LoadString(IDS_SELECT_CERTIFICATE_ISSUEDTO))
    {
      m_ctrlList.InsertColumn(nColumn, sColumnHeading, LVCFMT_LEFT, nColumnWidth+16);
      ++nColumn;
    }
  }
  if (!m_bExcludeIssuedByColumn)
  {
    if (sColumnHeading.LoadString(IDS_SELECT_CERTIFICATE_ISSUEDBY))
    {
      m_ctrlList.InsertColumn(nColumn, sColumnHeading, LVCFMT_LEFT, nColumnWidth);
      ++nColumn;
    }
  }
  if (!m_bExcludeIntendedPurposesColumn)
  {
    if (sColumnHeading.LoadString(IDS_SELECT_CERTIFICATE_INTENDEDPURPOSES))
    {
      m_ctrlList.InsertColumn(nColumn, sColumnHeading, LVCFMT_LEFT, nColumnWidth);
      ++nColumn;
    }
  }
  if (!m_bExcludeFriendlyNameColumn)
  {
    if (sColumnHeading.LoadString(IDS_SELECT_CERTIFICATE_FRIENDLYNAME))
    {
      m_ctrlList.InsertColumn(nColumn, sColumnHeading, LVCFMT_LEFT, nColumnWidth);
      ++nColumn;
    }
  }
  if (!m_bExcludeExpirationColumn)
  {
    if (sColumnHeading.LoadString(IDS_SELECT_CERTIFICATE_EXPIRATION))
    {
      m_ctrlList.InsertColumn(nColumn, sColumnHeading, LVCFMT_LEFT, nColumnWidth);
      ++nColumn;
    }
  }

  //Note unlike the standard CryptUIDlgSelectCertificate API call, CSelectCertificateDlg does
  //not support showing the "Location" column because I simply do not know what Windows
  //displays in this column. If anyone knows please let me know, and I will consider matching
  //its functionality.
}

CString CSelectCertificateDlg::SafeGetDateFormat(_In_ LCID locale, _In_ DWORD dwFlags, _In_ const SYSTEMTIME* pSystemTime)
{
  //What will be the return value from this function
  CString sInfo;

  //First ask the API to give us the buffer size
  int nChars{GetDateFormat(locale, dwFlags, pSystemTime, nullptr, nullptr, 0)};
  bool bWantStop{false};
  while (nChars && !bWantStop)
  {
    nChars = GetDateFormat(locale, dwFlags, pSystemTime, nullptr, sInfo.GetBuffer(nChars), nChars);
    sInfo.ReleaseBuffer();
    if (nChars == 0)
    {
      if (GetLastError() == ERROR_INSUFFICIENT_BUFFER)
      {
        //Expand the buffer, and loop around again
        nChars *= 2;
      }
      else
        bWantStop = true;
    }
    else
      bWantStop = true;
  }

  return sInfo;
}

CString CSelectCertificateDlg::SafeGetLocaleInfo(_In_ LCID Locale, _In_ LCTYPE LCType)
{
  //What will be the return value from this function
  CString sInfo;

  //First ask the API to give us the buffer size
  const int nChars{GetLocaleInfo(Locale, LCType, nullptr, 0)};
  if (nChars)
  {
    GetLocaleInfo(Locale, LCType, sInfo.GetBuffer(nChars), nChars);
    sInfo.ReleaseBuffer();
  }

  return sInfo;
}

void CSelectCertificateDlg::AddCertificateToUI(_In_ PCCERT_CONTEXT pCertContext, _In_ bool bSelected)
{
  //By default add the item to the UI, a derived class could inherit this 
  //function and decide to not add a specific certificate to the UI by 
  //simply not calling the base class implementation
  LVITEM item{};
  item.mask = LVIF_IMAGE | LVIF_TEXT | LVIF_STATE;
  int nItem{m_ctrlList.GetItemCount()};
  item.iItem = nItem;
#pragma warning(suppress: 26465 26492)
  item.pszText = const_cast<LPTSTR>(_T(""));
  item.iImage = 0;
  item.stateMask = LVIS_SELECTED;
  if (bSelected)
    item.state = LVIS_SELECTED;
  nItem = m_ctrlList.InsertItem(&item);

  //Also stash the item into the Certificates array
  m_Certificates.push_back(pCertContext);
  m_CertificateSelectionState.push_back(bSelected);

  //Now update all the column properties
  int nColumn{0};
  CString sColumnHeading;
  if (!m_bExcludeIssuedToColumn)
  {
    const DWORD dwChars{CertGetNameString(pCertContext, CERT_NAME_SIMPLE_DISPLAY_TYPE, 0, nullptr, nullptr, 0)};
    CString sBuffer;
    CertGetNameString(pCertContext, CERT_NAME_SIMPLE_DISPLAY_TYPE, 0, nullptr, sBuffer.GetBuffer(dwChars), dwChars);
    sBuffer.ReleaseBuffer();
    m_ctrlList.SetItemText(nItem, nColumn, sBuffer);
    ++nColumn;
  }
  if (!m_bExcludeIssuedByColumn)
  {
    const DWORD dwChars{CertGetNameString(pCertContext, CERT_NAME_SIMPLE_DISPLAY_TYPE, CERT_NAME_ISSUER_FLAG, nullptr, nullptr, 0)};
    CString sBuffer;
    CertGetNameString(pCertContext, CERT_NAME_SIMPLE_DISPLAY_TYPE, CERT_NAME_ISSUER_FLAG, nullptr, sBuffer.GetBuffer(dwChars), dwChars);
    sBuffer.ReleaseBuffer();
    m_ctrlList.SetItemText(nItem, nColumn, sBuffer);
    ++nColumn;
  }
  if (!m_bExcludeIntendedPurposesColumn)
  {
    //Call CertGetEnhancedKeyUsage initially to get the size of the buffer to allocate for it
    CString sIntendedUse;
    DWORD dwBufSize{0};
    CertGetEnhancedKeyUsage(pCertContext, 0, nullptr, &dwBufSize);
    if (dwBufSize)
    {
      //Get the list separator string
      CString sSeparator(SafeGetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SLIST));

      //Allocate some heap memory and call CertGetEnhancedKeyUsage again
      std::vector<BYTE> byCertEnhUsageDevices{dwBufSize, std::allocator<BYTE>{}};
#pragma warning(suppress: 26429 26489 26490)
      auto pCertEnhUsage{reinterpret_cast<PCERT_ENHKEY_USAGE>(byCertEnhUsageDevices.data())};
#pragma warning(suppress: 26486)
      CertGetEnhancedKeyUsage(pCertContext, 0, pCertEnhUsage, &dwBufSize);
#pragma warning(suppress: 26489)
      if (pCertEnhUsage->cUsageIdentifier)
      {
#pragma warning(suppress: 26489)
        for (DWORD i{0}; i<pCertEnhUsage->cUsageIdentifier; i++)
        {
          CString sUse;
#pragma warning(suppress: 26481 26489)
          if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_PKIX_KP_SERVER_AUTH))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_SERVER_AUTHENTICATION))
              ASSERT(false);
          }
#pragma warning(suppress: 26481 26489)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_PKIX_KP_CLIENT_AUTH))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_CLIENT_AUTHENTICATION))
              ASSERT(false);
          }
#pragma warning(suppress: 26481 26489)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_PKIX_KP_CODE_SIGNING))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_CODE_SIGNING))
              ASSERT(false);
          }
#pragma warning(suppress: 26481 26489)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_PKIX_KP_EMAIL_PROTECTION))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_SECURE_EMAIL))
              ASSERT(false);
          }
#pragma warning(suppress: 26481 26489)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_PKIX_KP_TIMESTAMP_SIGNING))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_TIME_STAMPING))
              ASSERT(false);
          }
#pragma warning(suppress: 26481 26489)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_PKIX_KP_OCSP_SIGNING))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_SIGNING_OCSP_RESPONSES))
              ASSERT(false);
          }
#pragma warning(suppress: 26481 26489)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_PKIX_OCSP_NOCHECK))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_OCSP_NOCHECK))
              ASSERT(false);
          }
#pragma warning(suppress: 26481 26489)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_PKIX_OCSP_NONCE))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_OCSP_NONCE))
              ASSERT(false);
          }
#pragma warning(suppress: 26481 26489)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_TRUSTED_CODESIGNING_CA_LIST))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_MICROSOFT_TRUST_LIST_SIGNING))
              ASSERT(false);
          }
#pragma warning(suppress: 26481 26489)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_KP_CTL_USAGE_SIGNING))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_MICROSOFT_CTL_SIGNER))
              ASSERT(false);
          }
#pragma warning(suppress: 26481 26489)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_KP_TIME_STAMP_SIGNING))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_MICROSOFT_TIME_STAMPING))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_SERVER_GATED_CRYPTO))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_MICROSOFT_SERVER_GATED_CRYPTO))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_SGC_NETSCAPE))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_SGC_NETSCAPE))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_PKIX_KP_IPSEC_END_SYSTEM))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_IPSEC_END_SYSTEM))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_PKIX_KP_IPSEC_TUNNEL))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_IPSEC_TUNNEL_TERMINATION))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_PKIX_KP_IPSEC_USER))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_IPSEC_USER))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_KP_EFS))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_ENCRYPTING_FILE_SYSTEM))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_WHQL_CRYPTO))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_WINDOWS_HARDWARE_DRIVER_VERIFICATION))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_NT5_CRYPTO))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_WINDOWS_SYSTEM_COMPONENT_VERIFICATION))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_OEM_WHQL_CRYPTO))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_OEM_WINDOWS_SYSTEM_COMPONENT_VERIFICATION))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_EMBEDDED_NT_CRYPTO))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_EMBEDDED_WINDOWS_SYSTEM_COMPONENT_VERIFICATION))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_LICENSES))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_KEY_PACK_LICENSES))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_LICENSE_SERVER))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_LICENSE_SERVER_VERIFICATION))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_KP_SMARTCARD_LOGON))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_SMART_CARD_LOGON))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_KP_KERNEL_MODE_CODE_SIGNING))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_KERNEL_MODE_CODE_SIGNING))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_DRM))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_DIGITAL_RIGHTS))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_DRM_INDIVIDUALIZATION))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_DRM_INDIVIDUALIZATION))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_KP_QUALIFIED_SUBORDINATION))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_QUALIFIED_SUBORDINATION))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_KP_KEY_RECOVERY))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_KEY_RECOVERY))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_KP_DOCUMENT_SIGNING))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_DOCUMENT_SIGNING))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_IPSEC_KP_IKE_INTERMEDIATE))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_IPSEC_IKE_INTERMEDIATE))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_EFS_RECOVERY))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_FILE_RECOVERY))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_ROOT_LIST_SIGNER))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_ROOT_LIST_SIGNER))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_ANY_APPLICATION_POLICY))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_ALL_APPLICATION_POLICIES))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_DS_EMAIL_REPLICATION))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_DIRECTORY_SERVICE_EMAIL_REPLICATION))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_REQUEST_CLIENT_INFO))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_CERTIFICATE_REQUEST_AGENT))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_KP_KEY_RECOVERY_AGENT))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_KEY_RECOVERY_AGENT))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_KP_CA_EXCHANGE))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_CA_ENCRYPTION_CERTIFICATE))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_KP_LIFETIME_SIGNING))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_LIFETIME_SIGNING))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_KP_MOBILE_DEVICE_SOFTWARE))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_MOBILE_DEVICE_SOFTWARE))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_KP_SMART_DISPLAY))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_SMART_DISPLAY))
              ASSERT(false);
          }
#pragma warning(suppress: 26481)
          else if (!strcmp(pCertEnhUsage->rgpszUsageIdentifier[i], szOID_KP_CSP_SIGNATURE))
          {
            if (!sUse.LoadString(IDS_SELECT_CERTIFICATE_CSP_SIGNATURE))
              ASSERT(false);
          }

          if (i == 0)
            sIntendedUse = sUse;
          else
          {
            sIntendedUse += sSeparator;
            sIntendedUse += sUse;
          }
        }
      }
      else
      {
        if (!sIntendedUse.LoadString(IDS_SELECT_CERTIFICATE_ALLINTENDEDUSE))
          ASSERT(false);
      }
    }
    else
    {
      if (!sIntendedUse.LoadString(IDS_SELECT_CERTIFICATE_ALLINTENDEDUSE))
        ASSERT(false);
    }

    m_ctrlList.SetItemText(nItem, nColumn, sIntendedUse);
    ++nColumn;
  }
  if (!m_bExcludeFriendlyNameColumn)
  {
    DWORD dwBufferSize{0};
    CertGetCertificateContextProperty(pCertContext, CERT_FRIENDLY_NAME_PROP_ID, nullptr, &dwBufferSize);
    if (dwBufferSize)
    {
      CStringW sBuffer;
      CertGetCertificateContextProperty(pCertContext, CERT_FRIENDLY_NAME_PROP_ID, sBuffer.GetBuffer(dwBufferSize/sizeof(wchar_t)), &dwBufferSize);
      sBuffer.ReleaseBuffer();
      m_ctrlList.SetItemText(nItem, nColumn, CString{sBuffer});
    }

    ++nColumn;
  }
  if (!m_bExcludeExpirationColumn)
  {
    //Get the expiry details of the certificate
    SYSTEMTIME st{};
#pragma warning(suppress: 26489)
    FileTimeToSystemTime(&pCertContext->pCertInfo->NotAfter, &st);
    m_ctrlList.SetItemText(nItem, nColumn, SafeGetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &st));
    ++nColumn;
  }
}

void CSelectCertificateDlg::AddCertificate(_In_ PCCERT_CONTEXT pCertContext, _In_ int nIndex)
{
  //If we have an initial selected cert, then try to show it as selected, otherwise
  //select the first certificate shown. This second option emulates the behaviour
  //of "CryptUIDlgSelectCertificate". A derived class is free to implement their
  //own version of AddCertificate and change this behaviour.
  if (m_pInitialSelectedCertificate != nullptr)
#pragma warning(suppress: 26486)
    AddCertificateToUI(pCertContext, CertCompareCertificate(X509_ASN_ENCODING | PKCS_7_ASN_ENCODING, m_pInitialSelectedCertificate->pCertInfo, pCertContext->pCertInfo));
  else
    AddCertificateToUI(pCertContext, (nIndex == 0));
}

void CSelectCertificateDlg::EnumerateCertificates(_In_ HCERTSTORE hCertStore)
{
  //Enumerate all the certificates in the specified store
  ASSERT(m_hCertStore);
  int nIndex{0};
  PCCERT_CONTEXT pCertContext{CertEnumCertificatesInStore(hCertStore, nullptr)};
  while (pCertContext != nullptr)
  {
    //Call the function to add the certificate to the UI
#pragma warning(suppress: 26486)
    PCCERT_CONTEXT pCopyCertContext{CertDuplicateCertificateContext(pCertContext)};
#pragma warning(suppress: 26486)
    AddCertificate(pCopyCertContext, nIndex);

    //Prepare for the next loop around
    ++nIndex;
    pCertContext = CertEnumCertificatesInStore(m_hCertStore, pCertContext);
  }
}

void CSelectCertificateDlg::SetupViewButtonState()
{
  //Hide the View Certificate button if required
  if (m_bHideViewCertificateButton)
  {
    CWnd* pViewButton{GetDlgItem(IDC_VIEW_CERTIFICATE)};
#pragma warning(suppress: 26496)
    AFXASSUME(pViewButton != nullptr);
    pViewButton->ShowWindow(SW_HIDE);

    //Move the OK and Cancel buttons over the now hidden "View Certificate" button
    CRect rCancel;
    CWnd* pCancelButton{GetDlgItem(IDCANCEL)};
#pragma warning(suppress: 26496)
    AFXASSUME(pCancelButton != nullptr);
    pCancelButton->GetWindowRect(&rCancel);
    ScreenToClient(&rCancel);
    CRect rView;
    pViewButton->GetWindowRect(&rView);
    ScreenToClient(&rView);
    const int nDeltaX{rView.right - rCancel.right};
    pCancelButton->MoveWindow(rCancel.left + nDeltaX, rCancel.top, rCancel.Width(), rCancel.Height());
    CWnd* pOKButton{GetDlgItem(IDOK)};
#pragma warning(suppress: 26496)
    AFXASSUME(pOKButton != nullptr);
    CRect rOK;
    pOKButton->GetWindowRect(&rOK);
    ScreenToClient(&rOK);
    pOKButton->MoveWindow(rOK.left + nDeltaX, rOK.top, rOK.Width(), rOK.Height());
  }
}

BOOL CSelectCertificateDlg::OnInitDialog()
{
  //Let the base class do its thing
  __super::OnInitDialog();

  //Call the various functions to setup the dialog correctly
  SetupListControl();
  UpdateCaptionAndPromptStrings();
  SetupViewButtonState();
  AddCertificateColumns();
  EnumerateCertificates(m_hCertStore);

  return TRUE;
}

#pragma warning(suppress: 26434)
void CSelectCertificateDlg::OnDestroy()
{
  //Get the selection state of all the certificates we have shown
  const int nCertificates{m_ctrlList.GetItemCount()};
  ASSERT(nCertificates == static_cast<int>(m_Certificates.size()));
  ASSERT(nCertificates == static_cast<int>(m_CertificateSelectionState.size()));
  for (int i{0}; i<nCertificates; i++)
  {
    const bool bSelected{m_ctrlList.GetItemState(i, LVIS_SELECTED) != 0};
#pragma warning(suppress: 26446)
    m_CertificateSelectionState[i] = bSelected;
  }

  //Let the base class do its thing
  __super::OnDestroy();
}

LRESULT CSelectCertificateDlg::OnKickIdle(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
  //Cause the OnUpdate MFC UI mechanism to work in a dialog
  UpdateDialogControls(this, TRUE);

  return 0;
}

#pragma warning(suppress: 26429)
void CSelectCertificateDlg::OnUpdateViewCertificate(CCmdUI* pCmdUI)
{
  //We can only view a certificate if there is something selected in the certificates list control
#pragma warning(suppress: 26486)
  pCmdUI->Enable(m_ctrlList.GetNextItem(-1, LVNI_SELECTED) != -1);
}

void CSelectCertificateDlg::OnViewCertificate()
{
  //Get the selected certificate and show it
  const int nItem{m_ctrlList.GetNextItem(-1, LVNI_SELECTED)};
  if (nItem != -1)
  {
    CRYPTUI_VIEWCERTIFICATE_STRUCT cuivc;
    memset(&cuivc, 0, sizeof(cuivc));
    cuivc.dwSize = sizeof(cuivc);
    cuivc.hwndParent = GetSafeHwnd();
    cuivc.dwFlags = m_dwViewFlags;
#pragma warning(suppress: 26446)
    cuivc.pCertContext = m_Certificates[nItem];
    BOOL bPropertiesChanged{FALSE};
#pragma warning(suppress: 26486)
    CryptUIDlgViewCertificate(&cuivc, &bPropertiesChanged);
  }
}
