/*
Module : SelectCertificateDlg.h
Purpose: Interface for a MFC dialog class to allow selection of cryptography certificates
Created: PJN / 27-08-2006

Copyright (c) 2006 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once

#ifndef __SELECTCERTIFICATEDLG_H__
#define __SELECTCERTIFICATEDLG_H__

#ifndef CSELECTCERTIFICATEDLG_EXT_CLASS
#define CSELECTCERTIFICATEDLG_EXT_CLASS
#endif //#ifndef CSELECTCERTIFICATEDLG_EXT_CLASS


//////////////////// Includes /////////////////////////////////////////////////

#ifndef __WINCRYPT_H__
#pragma message("To avoid this message, please put WinCrypt.h in your pre compiled header (normally stdafx.h)")
#include <WinCrypt.h>
#endif //#ifndef __WINCRYPT_H__

#ifndef WINTRUST_H
#pragma message("To avoid this message, please put WinTrust.h in your pre compiled header (normally stdafx.h)")
#include <WinTrust.h>
#endif //#ifndef WINTRUST_H

#ifndef _VECTOR_
#pragma message("To avoid this message, please put vector in your pre compiled header (normally stdafx.h)")
#include <vector>
#endif //#ifndef _VECTOR_


//////////////////// Classes //////////////////////////////////////////////////

class CSELECTCERTIFICATEDLG_EXT_CLASS CSelectCertificateDlg : public CDialog
{
public:
//Constructors / Destructors
  CSelectCertificateDlg(_In_opt_ CWnd* pParent = nullptr);
  CSelectCertificateDlg(_In_ const CSelectCertificateDlg&) = delete;
  CSelectCertificateDlg(_In_ CSelectCertificateDlg&&) = delete;
  ~CSelectCertificateDlg(); //NOLINT(modernize-use-override)

//Member variables
  bool m_bExcludeIssuedToColumn; //Do not show the "Issued to" column
  bool m_bExcludeIssuedByColumn; //Do not show the "Issued by" column
  bool m_bExcludeIntendedPurposesColumn; //Do not show the "Intended Purposes" column 
  bool m_bExcludeFriendlyNameColumn; //Do not show the "Friendly name" column
  bool m_bExcludeExpirationColumn; //Do not show the "Expiration" column
  HCERTSTORE m_hCertStore; //The certificate store to display the certificates for. Note client code is responsible for calling "CertCloseStore" on this
  CString m_sTitle; //The dialog caption to use. If left empty, the default title "Select certificate" will be used 
  CString m_sDisplayString; //The prompt text to use. If left empty, the default phrase "Select a certificate you want to use." will be used
  DWORD m_dwViewFlags; //The flags to pass to the "CryptUIDlgViewCertificate" function
  bool m_bAllowMultipleSelection; //Should we allow the user to select multiple certificates
  bool m_bHideViewCertificateButton; //Do not show the "View Certificate" button

//Methods
  CSelectCertificateDlg& operator=(_In_ const CSelectCertificateDlg&) = delete;
  CSelectCertificateDlg& operator=(_In_ CSelectCertificateDlg&&) = delete;
  _NODISCARD INT_PTR GetCertificatesShownCount() const noexcept; //Returns the total number of certificates which were shown.
  PCCERT_CONTEXT GetCertificateShown(_In_ INT_PTR nIndex, _Out_ bool& bWasSelected); //Returns the specific certificate which was shown at the specified index along with its final selection state. Note client 
                                                                                     //code is responsible for calling "CertFreeCertificateContext" for the returned cert context
  PCCERT_CONTEXT GetNextSelectedCertificateShown(_Inout_ int& nIndex); //Returns the next selected cert in the list, On input, nIndex specifies the index to start the search at, specify 0 to 
                                                                       //start at the beginning and upon return this value, will be the index of the next selected cert found. To continue the 
                                                                       //enumeration, simply increment this value by 1 and keep calling the function until it returns a nullptr. Note client 
                                                                       //code is responsible for calling "CertFreeCertificateContext" for the returned cert context
  bool SetInitialSelectedCert(_In_ PCCERT_CONTEXT pCertContext) noexcept; //Specifies the initial certificate to select. Note internally the class duplicates the cert context by calling 
                                                                          //"CertDuplicateCertificateContext" for it, so client code must still call "CertFreeCertificateContext" on the input parameter

protected:
//Enums
  enum { IDD = IDD_SELECT_CERTIFICATE_DLG };

//Member variables
  CStatic m_ctrlPrompt;
  CListCtrl m_ctrlList;
  CImageList m_ilCertificates;
  std::vector<PCCERT_CONTEXT> m_Certificates; //The certificates we are showing / have shown
  std::vector<bool> m_CertificateSelectionState; //Was the certificate selected, there is a 1-1 correspondence between this array 
                                                 //and m_Certificates
  PCCERT_CONTEXT m_pInitialSelectedCertificate; //The initial certificate to select in the dialog

//Virtual functions
  void DoDataExchange(CDataExchange* pDX) override;
  virtual void UpdateCaptionAndPromptStrings();
  virtual void AddCertificateColumns();
  virtual void AddCertificate(_In_ PCCERT_CONTEXT pCertContext, _In_ int nIndex);
  virtual void AddCertificateToUI(_In_ PCCERT_CONTEXT pCertContext, _In_ bool bSelected);
  virtual void EnumerateCertificates(_In_ HCERTSTORE hCertStore);
  virtual void SetupListControl();
  virtual void SetupViewButtonState();
  BOOL OnInitDialog() override;

//Static methods
  static CString SafeGetDateFormat(_In_ LCID locale, _In_ DWORD dwFlags, _In_ const SYSTEMTIME* pSystemTime);
  static CString SafeGetLocaleInfo(_In_ LCID Locale, _In_ LCTYPE LCType);

//Message handlers
  afx_msg void OnDestroy();
  afx_msg LRESULT OnKickIdle(WPARAM wParam, LPARAM lParam);
  afx_msg void OnViewCertificate();
  afx_msg void OnUpdateViewCertificate(CCmdUI* pCmdUI);

  DECLARE_MESSAGE_MAP()
};

#endif //#ifndef __SELECTCERTIFICATEDLG_H__
